/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-makefile-parser.h"

gboolean openldev_makefile_parser_is_blank (string);
void openldev_makefile_parser_add_attr (Makefile*, string);
void openldev_makefile_parser_add_array (Makefile*, string&);
void openldev_makefile_parser_add_script (Makefile*, string);
void openldev_makefile_parser_add_string (Makefile*, string);
void openldev_makefile_parser_add_comment (Makefile*, string);

// Create a new makefile object from the specified file
Makefile* openldev_makefile_parser_new (const char *file)
{
  Makefile *mf = new Makefile;
  mf->filename = file;
  string line;
  gboolean usecurrent = FALSE;
  
  // Loop for every line in the Makefile.am
  ifstream fin (file);
  while (getline (fin, line))
  {
    // If usecurrent==TRUE, that means that there is a script or
    //  array that may have another line
    if (usecurrent)
    {
      if (mf->types[mf->types.size()-1] == MAKEFILE_SCRIPT)
      {
        // Add the script if the line begins with a tab
        if (line[0] == 9)
        {
          line.erase (0, 1);
          mf->attributes[mf->attributes.size()-1]->attr.push_back(line);
        }
        // Otherwise, fall through
        else
          usecurrent = FALSE;
      }
      else if (mf->types[mf->types.size()-1] == MAKEFILE_ARRAY)
      {
        openldev_makefile_parser_add_attr (mf, line);
        
        // Stop any parsing if this is the last line of the array
        if (line.find ("\\", 0) == string::npos)
        {
          usecurrent = FALSE;
          line = "DO NOT PARSE";
        }
      }
      // Just a catch for odd cases
      else
        usecurrent = FALSE;
    }

    // This is used to parse all base elements
    if (!usecurrent && line != "DO NOT PARSE")
    {
      if (line[0] == '#')
        openldev_makefile_parser_add_comment (mf, line);
      else if (line.find("=", 0) != string::npos)
      {
        int L = line.find("\\", 0);
        if (L != (signed)string::npos)
        {
          if (L != (signed)line.length()-1)
          {
            if ((line[L+1] == ' ') || (line[L+1] == 9))
              usecurrent = TRUE;
          }
          else
            usecurrent = TRUE;
        }
        
        openldev_makefile_parser_add_array (mf, line);
        openldev_makefile_parser_add_attr (mf, line);
      }
      else if (line.find(":", 0) != string::npos)
      {
        openldev_makefile_parser_add_script (mf, line);
        usecurrent = TRUE;
      }
      else if (!openldev_makefile_parser_is_blank (line))
        openldev_makefile_parser_add_string (mf, line);
    }
  }
  
  fin.close ();
  return mf;
}

// Write the makefile with all of the changes. The default formatting is used.
// It is recommended to continue using this formatting if you make manual changes
// because OpenLDev will work with the file faster.
void openldev_makefile_parser_write (Makefile *mf)
{
  string content;
  unsigned int i, j;
  
  for (i=0; i < mf->types.size(); i++)
  {
    switch (mf->types[i])
    {
      case (MAKEFILE_STRING):
      {
        content += mf->names[i] + "\n";
        break;
      }
      case (MAKEFILE_COMMENT):
      {
        content += mf->names[i] + "\n";
        break;
      }
      case (MAKEFILE_SCRIPT):
      {
        if (mf->attributes[i]->attr.size() > 0)
        {
          content += mf->names[i] + "\n";
          for (j=0; j < mf->attributes[i]->attr.size(); j++)
            content += "\t" + mf->attributes[i]->attr[j] + "\n";
        }
        break;
      }
      case (MAKEFILE_ARRAY):
      {
        
        if (mf->attributes[i]->attr.size() > 0)
        {
          content += mf->names[i] + " = \\\n";
          for (j=0; j < mf->attributes[i]->attr.size()-1; j++)
            content += "\t" + mf->attributes[i]->attr[j] + "\\\n";
          content += "\t" + mf->attributes[i]->attr[mf->attributes[i]->attr.size()-1] + "\n";
        }
        break;
      }
      default:
        break;
    }
    
    content += "\n";
  }
  
  // Write the file
  ofstream fout (mf->filename.c_str());
  fout << content;
  fout.close ();
}

// Get a list of all of the sub-directories in the project. 
// This is recursive so it goes through all makefiles.
vector<string> openldev_makefile_parser_get_subdirs (Makefile *mf)
{
  for (unsigned int i=0; i < mf->types.size(); i++)
    if ((mf->types[i] == MAKEFILE_ARRAY) &&
        (mf->names[i] == "SUBDIRS"))
      return mf->attributes[i]->attr;
  
  vector<string> empty;
  return empty;
}

// Get a list of all of the sources in the makefile
vector<string> openldev_makefile_parser_get_sources (Makefile *mf)
{
  vector<string> sources;
  for (unsigned int i=0; i < mf->types.size(); i++)
    if ((mf->types[i] == MAKEFILE_ARRAY) &&
        (mf->names[i].find("SOURCES", 0) != string::npos))
      for (unsigned int j=0; j < mf->attributes[i]->attr.size(); j++)
        sources.push_back (mf->attributes[i]->attr[j]);
  
  return sources;
}

// Check if the current line is blank
gboolean openldev_makefile_parser_is_blank (string line)
{
  for (unsigned int i=0; i < line.length(); i++)
    if ((line[i] != ' ') && (line[i] != 9))
      return FALSE;
  
  return TRUE;
}

// Add a new array element and all of its attributes
void openldev_makefile_parser_add_array (Makefile *mf,
                                         string &line)
{
  string name = line;
  name = line.substr (0, line.find("=", 0));
  line.erase (0, line.find("=", 0)+1);
  while (name.find(" ", 0) != string::npos)
    name.erase (name.find(" ", 0), 1); 

  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_ARRAY);
  mf->attributes.push_back (obj);
  mf->names.push_back (name);
}

// Add an attribute for and array or a script
void openldev_makefile_parser_add_attr (Makefile *mf,
                                        string line)
{
  if (line == "DO NOT PARSE")
    return;

  // Clean up all slashes and front whitespace
  int L = line.find ("\\", 0);
  while (L != (signed)string::npos)
  {
    if (L != (signed)line.length()-1)
    {
      if ((line[L+1] == ' ') || (line[L+1] == 9))
        line.erase (L, 1);
    }
    else
      line.erase (L, 1);
    
    L = line.find ("\\", L+1);
  }
  while ((line[0] == ' ') || (line[0] == 9))
    line.erase (0, 1);
  for (unsigned int i=0; i < line.length(); i++)
    if (line[i] == 9)
      line[i] = ' ';
  
  // If the line is not empty, then add the element
  while (!line.empty())
  {
    if (line.find(" ", 0) != string::npos)
    {
      string attr = line.substr (0, line.find(" ", 0));
      if (!attr.empty())
        mf->attributes[mf->attributes.size()-1]->attr.push_back (attr);
      line.erase (0, line.find(" ", 0) + 1);
    }
    else
    {
      mf->attributes[mf->attributes.size()-1]->attr.push_back (line);
      line = "\0";
    }
  }
}

// Add a comment to the list of makefile elements
void openldev_makefile_parser_add_comment (Makefile *mf,
                                           string line)
{
  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_COMMENT);
  mf->attributes.push_back (obj);
  mf->names.push_back (line);
}

// Add a script to the makefile elements and 
// all of its content as attributes
void openldev_makefile_parser_add_script (Makefile *mf,
                                          string line)
{
  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_SCRIPT);
  mf->attributes.push_back (obj);
  mf->names.push_back (line);
}

// Add any other line as a string
void openldev_makefile_parser_add_string (Makefile *mf,
                                          string line)
{
  MakefileAttr *obj = new MakefileAttr;
  mf->types.push_back (MAKEFILE_STRING);
  mf->attributes.push_back (obj);
  mf->names.push_back (line);
}
