/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "openldev-browser-project.h"

vector<string> read_directory (const char*, gboolean);
void populate_browser_project (BrowserProject*, vector<string>);

// Create a new file browser object that will be added to the Browser
BrowserProject* openldev_browser_project_new ()
{
  BrowserProject *browser = new BrowserProject ();
 
  browser->treeview = gtk_tree_view_new ();
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW(browser->treeview), FALSE);
  
  browser->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (browser->scrolled_window), 
                                         browser->treeview);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (browser->scrolled_window), 
                                  GTK_POLICY_AUTOMATIC, 
                                  GTK_POLICY_ALWAYS);
  
  return browser;
}

// Get the name of the file that is currently selected along with the path
const char* openldev_browser_project_get_selected_file (BrowserProject* browser)
{
  GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(browser->treeview));
  GtkTreeModel *model;
  GtkTreeIter iter;

  if (gtk_tree_selection_get_selected (selection, &model, &iter))
  {
    gchar *fn;
    gtk_tree_model_get (model, &iter, 0, &fn, -1);
   
    string file = browser->base + browser->current + fn;
    return file.c_str ();
  }
  
  return "NONE";
}

// Populate the file browser with the list of files for the directory
void openldev_browser_project_populate (BrowserProject *browser, 
                                        const char *base_directory)
{
  if (string(base_directory) == "NULL")
    return;
  
  browser->base = base_directory;
  browser->current = "";
  if (browser->base[browser->base.length()-1] != '/')
    browser->base += "/";
  
  vector<string> filenames = read_directory (base_directory, FALSE);
  populate_browser_project (browser, filenames);
  filenames.clear();
}

// Move up one level in the file browser
void openldev_browser_project_move_up (BrowserProject *browser)
{
  // This will allow the stopping of moving the browser beyond its base
  if (browser->current.find("/", 0) != string::npos)
  {
    browser->current.erase (browser->current.length()-1, 1);
    while (browser->current.length() > 0 &&
           browser->current[browser->current.length()-1] != '/')
      browser->current.erase (browser->current.length()-1, 1);
  }
  
  // Repopulate with the new sources
  string dir = browser->base + browser->current;
  vector<string> filenames;
  if (browser->current.length() > 0)
    filenames = read_directory (dir.c_str(), TRUE);
  else
    filenames = read_directory (dir.c_str(), FALSE);
  
  populate_browser_project (browser, filenames);
  filenames.clear();
}

// Move to a specific new directory
void openldev_browser_project_move (BrowserProject *browser,
                                    const char* dir_name)
{
  browser->current += dir_name + string("/");
  string dir = browser->base + browser->current;
  vector<string> filenames = read_directory (dir.c_str(), TRUE);
  populate_browser_project (browser, filenames);
  filenames.clear();
}

// Refresh the current list of sources without changing the directory.
void openldev_browser_project_refresh (BrowserProject *browser)
{
  string dir = browser->base + browser->current;
  vector<string> filenames = read_directory (dir.c_str(), !(dir == browser->base));   
  populate_browser_project (browser, filenames);
  filenames.clear();
}

// Read the directory from the file system and populate the file browser
// with a list of all of the files in that directory.
vector<string> read_directory (const char *dir,
                               gboolean show_up)
{
  vector<string> files;
  struct dirent **namelist;
  int n = scandir(dir, &namelist, 0, alphasort), i;
  
  // Get the list of files by parsing through the list
  if (n >= 0)
  {
    while (n--) 
    {
      if (string(namelist[n]->d_name) != "." &&
          string(namelist[n]->d_name) != "..")
        files.push_back (namelist[n]->d_name);
      free (namelist[n]);
    } 
    free (namelist);
  }
  
  // Sort the vector in the opposite direction
  for (i=0; i<(signed)files.size()/2; i++)
  {
    string temp = files[i];
    files[i] = files[files.size()-i-1];
    files[files.size()-i-1] = temp;
  }
  
  // Insert the ability to move up a directory if the function
  // it told that it should do so.
  if (show_up)
    files.insert (files.begin(), "..");
  
  return files;
}

// Actually do the population of the file browser (finally)
void populate_browser_project (BrowserProject *browser,
                               vector<string> filenames)
{
  // Populate the view
  gnome_vfs_init ();
  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
  GtkTreeIter row;
  GtkTreeModel *model = GTK_TREE_MODEL (gtk_tree_store_new(2, GDK_TYPE_PIXBUF, G_TYPE_STRING));
                                                          
  gtk_tree_view_set_model (GTK_TREE_VIEW(browser->treeview), model);
  model = gtk_tree_view_get_model (GTK_TREE_VIEW(browser->treeview));
  
  while (g_list_length(gtk_tree_view_get_columns (GTK_TREE_VIEW(browser->treeview))) > 0)
    gtk_tree_view_remove_column (GTK_TREE_VIEW(browser->treeview),
      gtk_tree_view_get_column (GTK_TREE_VIEW(browser->treeview), 0));

  GtkCellRenderer *irenderer = gtk_cell_renderer_pixbuf_new ();
	GtkTreeViewColumn *icolumn = gtk_tree_view_column_new_with_attributes ("",
                               irenderer, "pixbuf", 0, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(browser->treeview), icolumn);
  
  GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
	GtkTreeViewColumn *column = gtk_tree_view_column_new_with_attributes ("Project Files",
                              renderer, "text", 1, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW(browser->treeview), column);
  
  for (unsigned int i=0; i < filenames.size(); i++)
  {
    string fn = browser->base + browser->current + filenames[i];
    gchar *mime = gnome_vfs_get_mime_type (fn.c_str());
    gchar *icon = gnome_icon_lookup (icon_theme, NULL, NULL, NULL, NULL,
                  mime, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
    GdkPixbuf *pixbuf = gtk_icon_theme_load_icon (icon_theme, icon, 16,
                        GTK_ICON_LOOKUP_NO_SVG, NULL);
    
    gtk_tree_store_append (GTK_TREE_STORE(model), &row, NULL);
    gtk_tree_store_set (GTK_TREE_STORE(model), &row, 0, pixbuf,
                        1, filenames[i].c_str(), -1);
  }
  
  if (filenames.size() > 0)
  {
    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(browser->treeview));
    gtk_tree_model_get_iter (model, &row, gtk_tree_path_new_from_string("0"));
    gtk_tree_selection_select_iter (selection, &row);
  }
  
  g_object_unref (model);
}
